import { isDefined } from '@collectable/core';
import { RedBlackTreeIterator, RedBlackTreeKeyIterator, RedBlackTreeValueIterator, PathNode, isNone } from '../internals';
/**
 * Retrieves the first entry in the tree, or undefined if the tree is empty.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(RedBlackTreeEntry<K, V>|undefined)} The first entry in the tree, or undefined if the tree is empty
 */
export function first(tree) {
    if (tree._size === 0)
        return void 0;
    var node = tree._root;
    while (!isNone(node._left)) {
        node = node._left;
    }
    return node;
}
/**
 * Retrieves the first key in the tree, or undefined if the tree is empty.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(K|undefined)} The first key in the tree, or undefined if the tree is empty
 */
export function firstKey(tree) {
    var node = first(tree);
    return isDefined(node) ? node.key : void 0;
}
/**
 * Retrieves the value of the first entry in the tree, or undefined if the tree is empty.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(K|undefined)} The value of the first entry in the tree, or undefined if the tree is empty
 */
export function firstValue(tree) {
    var node = first(tree);
    return isDefined(node) ? node.value : void 0;
}
/**
 * Returns an iterator that starts from the first entry in the tree and iterates toward the end of the tree. Emissions
 * are references to nodes in the tree, exposed directly to allow Collectable.RedBlackTree to be efficiently consumed as
 * a backing structure for other data structures. Do not modify the returned node.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {RedBlackTreeIterator<K, V>} An iterator for entries in the tree
 */
export function iterateFromFirst(tree) {
    var path = PathNode.NONE;
    var node = tree._root;
    while (!isNone(node)) {
        path = PathNode.next(node, path, 1 /* LEFT */);
        node = node._left;
    }
    return RedBlackTreeIterator.create(path, tree._compare, false);
}
export function iterateValuesFromFirst(tree) {
    return new RedBlackTreeValueIterator(iterateFromFirst(tree));
}
export function iterateKeysFromFirst(tree) {
    return new RedBlackTreeKeyIterator(iterateFromFirst(tree));
}

//# sourceMappingURL=first.js.map
