import { Mutation } from '@collectable/core';
import { numericCompare, stringCompare, isUndefined } from '@collectable/core';
import { set } from './set';
import { keys as keysOf } from './unwrap';
import { isRedBlackTree, createTree } from '../internals';
export function fromNumericKeys(keys, mutability) {
    return fromKeys(numericCompare, keys, mutability);
}
export function fromStringKeys(keys, mutability) {
    return fromKeys(stringCompare, keys, mutability);
}
export function fromKeys(compare, keys, mutability) {
    const tree = Mutation.modify(createTree(compare, mutability));
    if (Array.isArray(keys)) {
        for (var i = 0; i < keys.length; i++) {
            set(keys[i], null, tree);
        }
    }
    else {
        const it = isRedBlackTree(keys) ? keysOf(keys) : keys[Symbol.iterator]();
        var current;
        while (!(current = it.next()).done) {
            set(current.value, null, tree);
        }
    }
    return Mutation.commit(tree);
}
export function fromPairsWithNumericKeys(pairs, mutability) {
    return fromPairs(numericCompare, pairs, mutability);
}
export function fromPairsWithStringKeys(pairs, mutability) {
    return fromPairs(stringCompare, pairs, mutability);
}
/**
 * Creates a new `RedBlackTree` from an array of key/value pairs (tuples). If no ComparatorFn function is supplied, keys
 * are compared using logical less-than and greater-than operations, which will generally only be suitable for numeric
 * or string keys.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {[K, V][]} pairs An array of pairs (tuples), each being a two-element array of [key, value]
 * @param {ComparatorFn<K>} compare A comparison function, taking two keys, and returning a value less than 0 if the
 *                                  first key is smaller than the second, a value greater than 0 if the first key is
 *                                  greater than the second, or 0 if they're the same.
 * @returns {RedBlackTreeStructure<K, V>} A tree populated with an entry for each pair in the input array
 */
export function fromPairs(compare, pairs, mutability) {
    const tree = Mutation.modify(createTree(compare, mutability));
    var pair;
    if (Array.isArray(pairs)) {
        for (var i = 0; i < pairs.length; i++) {
            pair = pairs[i];
            set(pair[0], pair[1], tree);
        }
    }
    else {
        const it = pairs[Symbol.iterator]();
        var current;
        while (!(current = it.next()).done) {
            pair = current.value;
            set(pair[0], pair[1], tree);
        }
    }
    return Mutation.commit(tree);
}
export function fromObject(obj, compare, mutability) {
    if (Mutation.isMutationContext(compare)) {
        mutability = compare;
        compare = void 0;
    }
    if (isUndefined(compare)) {
        compare = stringCompare;
    }
    const tree = Mutation.modify(createTree(compare, mutability));
    const keys = Object.keys(obj);
    for (var i = 0; i < keys.length; i++) {
        var key = keys[i];
        set(key, obj[key], tree);
    }
    return Mutation.commit(tree);
}

//# sourceMappingURL=from.js.map
